from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyDatedResourceModel


class ScriptTag(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.ScriptTag

    event = models.CharField(max_length = 16)
    src = models.URLField()

    class Meta:
        app_label = 'shopify_sync'



# Generated by Django 2.1.4 on 2018-12-20 17:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shopify_sync', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='carrierservice',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='collect',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='customcollection',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='customer',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='image',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='lineitem',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='metafield',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='option',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='order',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='product',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='scripttag',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='shop',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='smartcollection',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='variant',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='webhook',
            name='admin_graphql_api_id',
            field=models.CharField(default='', max_length=80),
            preserve_default=False,
        ),
    ]



from __future__ import unicode_literals

import shopify
import logging
from django.db import models

from .base import ShopifyDatedResourceModel
from .collect import Collect
from .image import Image
from .option import Option
from .variant import Variant
from .metafield import Metafield
from .session import activate_session

log = logging.getLogger(__name__)


class Product(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Product
    child_fields = {
        'images': Image,
        'variants': Variant,
        'options': Option,
        # 'metafields': Metafield,
    }

    body_html = models.TextField(default='')
    handle = models.CharField(max_length=255, db_index=True)
    product_type = models.CharField(max_length=255, db_index=True)
    published_at = models.DateTimeField(null=True)
    published_scope = models.CharField(max_length=64, default='global')
    tags = models.CharField(max_length=255, blank=True)
    template_suffix = models.CharField(max_length=255, null=True)
    title = models.CharField(max_length=255, db_index=True)
    vendor = models.CharField(max_length=255, db_index=True, null=True)

    class Meta:
        app_label = 'shopify_sync'

    @property
    def images(self):
        return Image.objects.filter(product_id=self.id)

    @property
    def collects(self):
        return Collect.objects.filter(product_id=self.id)

    @property
    def variants(self):
        return Variant.objects.filter(product_id=self.id)

    @property
    def options(self):
        return Option.objects.filter(product_id=self.id)

    @property
    def price(self):
        return (
            min([variant.price for variant in self.variants]),
            max([variant.price for variant in self.variants]),
        )

    @property
    def weight(self):
        return (
            min([variant.grams for variant in self.variants]),
            max([variant.grams for variant in self.variants]),
        )

    def _get_tag_list(self):
        # Tags are comma-space delimited.
        # https://help.shopify.com/api/reference/product#tags-property
        return self.tags.split(', ') if self.tags else []

    def _set_tag_list(self, tag_list):
        # we need to make sure tag_list is a list, if it is not we will make it
        # one and we will use join to save to tags. The idea is that tag_list
        # will match self.tags at all time. DOESN'T AUTO SAVE
        self.tags = ', '.join(tag_list if isinstance(tag_list, list) else [tag_list])
        return self.tags
    tag_list = property(_get_tag_list, _set_tag_list)

    def add_tag(self, tag):
        # Add a tag or a list of tags
        if tag:
            self.tag_list += tag if isinstance(tag, list) else [tag]

    def remove_tag(self, tag):
        # remove all instances of a tag or list of tags
        if tag:
            rm_list = tag if isinstance(tag, list) else [tag]
            self.tag_list = [tag_ for tag_ in self.tag_list if tag_ not in rm_list]

    def save(self, sync_meta=False, *args, **kwargs):
        with activate_session(self) as shopify_resource:
            # only want to sync the metafields if we have it set to true
            metafields = shopify_resource.metafields() if sync_meta else []
            for metafield in metafields:
                defaults = metafield.attributes
                defaults.update({'product': self, 'session': self.session})
                instance, created = Metafield.objects.update_or_create(id=defaults['id'],
                                                                       defaults=defaults)
                _new = "Created" if created else "Updated"
                log.debug("%s metafield for product %s <%s>" % (_new, self, instance))
        super(Product, self).save(*args, **kwargs)

    def __str__(self):
        return self.title



import logging

log = logging.getLogger(__name__)


def get_topic_model(topic, data):
    from .models import (CustomCollection, Customer, Order, Product, Shop,
                         SmartCollection, Metafield)

    """
    Return the model related to the given topic, if it's a valid topic
    permitted by theme settings. If the topic isn't permitted, or there's
    no rule mapping the given topic to a model, None is returned.
    """
    topic = topic.split('/')[0]
    mapping = {
        'collections': SmartCollection if 'rules' in data else CustomCollection,
        'products': Product,
        'customers': Customer,
        'orders': Order,
        'metafields': Metafield,
        'shop': Shop,
    }
    return mapping.get(topic, None)


def get_topic_action(topic):
    topic = topic.split('/')[1]
    mapping = {
        'create': 'sync_one',
        'update': 'sync_one'
    }
    return mapping.get(topic, None)


def webhook_received_handler(sender, domain, topic, data, **kwargs):
    """
    Signal handler to process a received webhook.
    """

    # Get the model related to the incoming topic and data.
    model = get_topic_model(topic, data)
    if model is None:
        assert "topic model does not exist"

    # Get the action related to the incoming topic.
    model_action = get_topic_action(topic)
    if model_action is None:
        assert "topic action does not exist"

    # Convert the incoming data to the relevant Shopify resource.

    msg = "Creating model '%s' from webhook data" % model.__name__
    log.info(msg)
    shopify_resource = model.shopify_resource_from_json(data)

    # Execute the desired action.
    if model_action == 'sync_one':
        print("This will print")
        instance = model.objects.sync_one(shopify_resource)
        print("This will not")
    else:
        assert "The model action has to me sync_one"



from __future__ import unicode_literals

from shopify_webhook.tests import WebhookTestCase
from datetime import datetime
from dateutil import parser
from decimal import Decimal


class SyncTestCase(WebhookTestCase):
    """
    Base class providing helpers for running synchronisation-based tests.
    """

    def assertSynced(self, session, data, model):
        """
        Check that the given data has been synchronised locally for the given user and model class.
        """

        instance = model.objects.get(id=data['id'], session=session)

        # Check all direct fields on the instance were synchronised across correctly.
        for field in model.get_default_fields():
            if field in data:
                expected_value = data[field]
                actual_value = getattr(instance, field)

                # If our value is a datetime, ensure expected value is a UTC datetime object too.
                if isinstance(actual_value, datetime):
                    expected_value = parser.parse(expected_value)
                    expected_value = expected_value.utctimetuple()
                    actual_value = actual_value.utctimetuple()

                # If our value is a Decimal, convert the expected value to Decimal.
                if isinstance(actual_value, Decimal):
                    expected_value = Decimal(expected_value)

                # Assert values are equal.
                if expected_value != actual_value:
                    print("Field '%s' expected value is '%s' not '%s'" % (field, expected_value, actual_value,))
                self.assertEqual(expected_value, actual_value)

        # Check all child fields on the instance were synchronised correctly.
        for child_field, child_model in model.get_child_fields().items():
            for child_data in data[child_field]:
                self.assertSynced(session, child_data, child_model)



from __future__ import unicode_literals

from .address import Address
from .carrier_service import CarrierService
from .collect import Collect
from .custom_collection import CustomCollection
from .customer import Customer
from .image import Image
from .line_item import LineItem
from .metafield import Metafield
from .option import Option
from .order import Order
from .product import Product
from .script_tag import ScriptTag
from .session import Session
from .shop import Shop
from .smart_collection import SmartCollection
from .variant import Variant
from .webhook import Webhook



from __future__ import unicode_literals

import shopify
from django.db import models
from jsonfield import JSONField
import json

from ..encoders import ShopifyDjangoJSONEncoder, empty_list
from pyactiveresource.connection import ResourceInvalid
from .base import ShopifyDatedResourceModel
from .session import activate_session
from .line_item import LineItem
from .customer import Customer
from .address import ShippingAddress


class Order(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Order
    related_fields = ['customer', 'shipping_address']
    r_fields = {
        'customer': Customer,
        'shipping_address': ShippingAddress,
    }
    child_fields = {
        'line_items': LineItem,
    }

    billing_address = JSONField(null=True, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})
    browser_ip = models.GenericIPAddressField(null = True)
    buyer_accepts_marketing = models.BooleanField(default = False)
    cancel_reason = models.CharField(max_length = 32, null = True)
    cancelled_at = models.DateTimeField(null = True)
    cart_token = models.CharField(max_length = 32, null = True)
    client_details = JSONField(dump_kwargs = {'cls': ShopifyDjangoJSONEncoder}, null=True)
    closed_at = models.DateTimeField(null = True)
    currency = models.CharField(max_length = 3)
    customer = models.ForeignKey('shopify_sync.Customer', null=True, on_delete=models.CASCADE)
    discount_codes = JSONField(default = empty_list, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder}, null=True)
    email = models.EmailField()
    financial_status = models.CharField(max_length = 32)
    fulfillment_status = models.CharField(max_length = 32, null = True)
    tags = models.TextField(null=True)
    landing_site = models.URLField(max_length=2048, null=True)
    name = models.CharField(max_length = 32)
    note = models.TextField(null = True)
    note_attributes = JSONField(dump_kwargs = {'cls': ShopifyDjangoJSONEncoder}, null=True)
    number = models.IntegerField()
    order_number = models.BigIntegerField()
    processed_at = models.DateTimeField()
    processing_method = models.CharField(max_length = 32)
    referring_site = models.URLField(max_length = 2048, null = True)
    shipping_address = models.ForeignKey('shopify_sync.ShippingAddress', null=True, related_name='shipping_address', on_delete=models.CASCADE)
    shipping_lines = JSONField(default = empty_list, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})
    source_name = models.CharField(max_length = 32)
    tax_lines = JSONField(default = empty_list, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder}, null=True)
    taxes_included = models.BooleanField(default = True)
    token = models.CharField(max_length = 32)
    total_discounts = models.DecimalField(max_digits = 10, decimal_places = 2)
    total_line_items_price = models.DecimalField(max_digits = 10, decimal_places = 2)
    total_price = models.DecimalField(max_digits = 10, decimal_places = 2)
    total_tax = models.DecimalField(max_digits = 10, decimal_places = 2)
    total_weight = models.DecimalField(max_digits = 10, decimal_places = 2)

    class Meta:
        app_label = 'shopify_sync'

    def fix_ids(self):
        for line_item in self.line_items:
            line_item.fix_ids()

    def _fulfillments(self):
        return []
    fulfillments = property(_fulfillments)

    def _line_items(self):
        return LineItem.objects.filter(order=self)
    line_items = property(_line_items)

    def calculate_refund(self, line_items=None):
        URL = 'refunds/calculate'
        line_items = line_items if line_items else self.line_items
        refund_lines = []
        for line in line_items:
            refund_lines.append({
                "line_item_id": line.id,
                "quantity": line.quantity
            })

        data = {
            "refund": {
               "shipping": {
                    "full_refund": True
                },
                "refund_line_items": refund_lines
            }
        }
        body = json.dumps(data)
        with activate_session(self) as shopify_resource:
            try:
                response = shopify_resource.post(URL, body=body)
            except ResourceInvalid as response:
                print("sent data %s" % body)
                return response

        body = json.loads(response.body)
        errors = data.pop('errors', None)
        if errors:
            return errors
        print(body)
        return body

    def refund_from_transaction(self, data):
        URL = 'refunds'

        errors = data.pop('errors', None)
        if errors:
            return errors

        transactions = data['refund'].pop('transactions')
        if not transactions:
            return "could not refund"
        for transaction in transactions:
            transaction.update({
                "kind": "refund",
            })
        data['refund'].update({
            "restock": False,
            "note": "The Campaign failed, sorry",
            "transactions": transactions,
        })

        body = json.dumps(data)
        with activate_session(self) as shopify_resource:
            response = shopify_resource.post(URL, body=body)
        body = json.loads(response.body)

        return body

    def refund(self, **kwargs):
        data = self.calculate_refund(**kwargs)
        if isinstance(data, dict):
            return self.refund_from_transaction(data)
        else:
            return data

    def _refunds(self):
        URL = "refunds"
        with activate_session(self) as shopify_resource:
            return shopify_resource.get(URL)
    refunds = property(_refunds)

    def __str__(self):
        return self.name



from unittest import TestCase
from ..models import Product


class ProductTagBaseCase(TestCase):
    def setUp(self):
        self.single_tag = "Car"
        self.multi_tag = ["Boat", "Duck"]
        self.multi_tag_str = ", ".join(self.multi_tag)
        self.prod_tags = "New, Old"
        self.prod_tag = "Green"
        self.product = Product()

    @staticmethod
    def assertContains(larger, smaller):
        for i in smaller.split(", "):
            assert i in larger, "'%s' is NOT in '%s'" % (i, larger)

    @staticmethod
    def assertNotContains(larger, smaller):
        for i in smaller.split(", "):
            assert i not in larger, "'%s' is in '%s'" % (i, larger)


class ProductSingleTagAddTestCase(ProductTagBaseCase):
    def test_add_single_tag_with_no_tags(self):
        self.assertEqual(self.product.tags, "")
        self.product.add_tag(self.single_tag)
        self.assertEqual(self.product.tags, self.single_tag)

    def test_add_single_tag_with_one_tag(self):
        self.product.tags = self.prod_tag
        self.product.add_tag(self.single_tag)
        self.assertContains(self.product.tags, self.single_tag)
        self.assertContains(self.product.tags, self.prod_tag)

    def test_add_single_tag_with_multi_tag(self):
        self.product.tags = self.prod_tags
        self.product.add_tag(self.single_tag)
        self.assertContains(self.product.tags, self.single_tag)
        self.assertContains(self.product.tags, self.prod_tags)


class ProductSingleTagRemoveTestCase(ProductTagBaseCase):
    def test_remove_single_tag_with_no_tags(self):
        self.assertEqual(self.product.tags, "")
        self.product.remove_tag(self.single_tag)
        self.assertEqual(self.product.tags, "")

    def test_remove_single_tag_with_one_tag(self):
        self.product.tags = self.prod_tag + ', ' + self.single_tag
        self.product.remove_tag(self.single_tag)
        self.assertNotContains(self.product.tags, self.single_tag)
        self.assertContains(self.product.tags, self.prod_tag)

    def test_remove_single_tag_with_multi_tag(self):
        self.product.tags = self.prod_tags + ', ' + self.single_tag
        self.product.remove_tag(self.single_tag)
        self.assertNotContains(self.product.tags, self.single_tag)
        self.assertContains(self.product.tags, self.prod_tags)


class ProductMultiTagAddTestCase(ProductTagBaseCase):
    def test_add_multi_tag_with_no_tags(self):
        self.assertEqual(self.product.tags, "")
        self.product.add_tag(self.multi_tag)
        self.assertEqual(self.product.tags, self.multi_tag_str)

    def test_add_multi_tag_with_one_tag(self):
        self.product.tags = self.prod_tag
        self.product.add_tag(self.multi_tag)
        self.assertContains(self.product.tags, self.multi_tag_str)
        self.assertContains(self.product.tags, self.prod_tag)

    def test_add_multi_tag_with_multi_tag(self):
        self.product.tags = self.prod_tags
        self.product.add_tag(self.multi_tag)
        self.assertContains(self.product.tags, self.multi_tag_str)
        self.assertContains(self.product.tags, self.prod_tags)


class ProductMultiTagRemoveTestCase(ProductTagBaseCase):
    def test_remove_multi_tag_with_no_tags(self):
        self.assertEqual(self.product.tags, "")
        self.product.remove_tag(self.multi_tag)
        self.assertEqual(self.product.tags, "")

    def test_remove_multi_tag_with_one_tag(self):
        self.product.tags = self.prod_tag + ', ' + self.multi_tag_str
        self.product.remove_tag(self.multi_tag)
        self.assertNotContains(self.product.tags, self.multi_tag_str)
        self.assertContains(self.prod_tag, self.product.tags)

    def test_remove_multi_tag_with_multi_tag(self):
        self.product.tags = self.prod_tags + ', ' + self.multi_tag_str
        self.product.remove_tag(self.multi_tag)
        self.assertNotContains(self.product.tags, self.multi_tag_str)
        self.assertContains(self.product.tags, self.prod_tags)



from __future__ import unicode_literals

from django.db import models
from shopify import Session as ShopifySession, ShopifyResource
from contextlib import contextmanager


class Session(models.Model):
    token = models.CharField(max_length=255)
    site = models.CharField(max_length=511)

    class Meta:
        app_label = 'shopify_sync'

    def to_shopify(self):
        shopify_session = ShopifySession(self.site, self.token)
        # pyactiveresource has a defined __setattr__
        shopify_session.__dict__['model'] = self
        shopify_session.__dict__['session'] = shopify_session
        return shopify_session

    def __str__(self):
        return "Session: %s" % self.site


@contextmanager
def activate_session(obj, session=None):
    """
    We want to make sure that we do not just use 'activate_session' and
    then not close the session, this is our solution.

    For this we take a Session, ShopifyResource, or model and return a
    ShopifyResource with the addition attrs of model and session.
    """

    if isinstance(obj, Session):
        shopify_resource = obj.to_shopify()

    elif isinstance(obj, ShopifyResource):
        shopify_resource = obj
        if session:
            # Use the session if we are given it and make sure it is a shopify
            # session we connect
            if isinstance(session, ShopifySession):
                shopify_resource.__dict__['session'] = session
            else:
                shopify_resource.__dict__['session'] = session.to_shopify()
        elif hasattr(shopify_resource, 'session'):
            # If there was no session provided, see if the resource has one
            # attached
            shopify_resource.activate_session(shopify_resource.session)
            try:
                yield shopify_resource
            except Exception as err:
                shopify_resource.clear_session()
                raise err
            else:
                shopify_resource.clear_session()
        else:
            # Otherwise we have to try find the session the session from the
            # resouce insides.
            print("Intorspection to find the session!", shopify_resource.__dict__)
            site = shopify_resource.connection._parse_site(obj.__class__.site)
            if site:
                # We can't do anything as there is no site given
                raise AttributeError("Object does not have a site attached. Please pass session")
            else:
                site = site[0].replace('https://', '')
                try:
                    session = Session.objects.get(site=site)
                except models.DoesNotExist:
                    raise models.DoesNotExist("The session for site '%s' does not exist. "
                                              "You must create a session first by having the "
                                              "site login first" % site)
                else:
                    shopify_resource.session = session.to_shopify()

    elif isinstance(obj, models.Model):
        shopify_resource = obj.to_shopify_resource()
        shopify_resource.session = obj.session.to_shopify()
        shopify_resource.model = obj

    elif hasattr(obj, 'session'):
        shopify_resource = obj
        shopify_resource.activate_session(obj.session)

    else:
        raise TypeError("Object needs to be a Model, ShopifyResource, or Session not '%s'." % type(obj))

    if not isinstance(shopify_resource, ShopifySession):
        # We now can activate the session if it isn't a ShopifySession
        shopify_resource.activate_session(shopify_resource.session)
    try:
        yield shopify_resource
    except Exception as err:
        shopify_resource.clear_session()
        raise err
    else:
        shopify_resource.clear_session()



from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyDatedResourceModel


class Image(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Image
    parent_field = 'product_id'

    position = models.IntegerField(null = True, default = 1)
    product = models.ForeignKey('shopify_sync.Product', on_delete=models.CASCADE)
    src = models.URLField()

    class Meta:
        app_label = 'shopify_sync'

    def __str__(self):
        return self.src



from __future__ import unicode_literals

import logging
from copy import copy
import math

from django.core.serializers.json import DjangoJSONEncoder
from django.db import models, utils
from django.db.models.fields.related import ForeignObjectRel
from pyactiveresource.connection import ResourceNotFound

from .. import SHOPIFY_API_PAGE_LIMIT

from .session import Session, ShopifyResource, ShopifySession, activate_session

log = logging.getLogger(__name__)


def get_shopify_pagination(total_count):
    """
    Get the appropriate pagination to use with Shopify's API
    given the total number of records.

    :param total_count: The number of resources that you are trying to get.
    :return:
    :rtype: tuple with the starting page, last page and the page limit.
    """
    last_page = (int(math.ceil(float(total_count) /
                 float(SHOPIFY_API_PAGE_LIMIT))))
    return (1, last_page, SHOPIFY_API_PAGE_LIMIT,)


class ChangedFields(object):
    """
    Keeps track of fields that have changed since model instantiation, and on
    save updates only those fields.

    If save is called with update_fields, the passed kwarg is given precedence.

    A caveat: This can't do anything to help you with ManyToManyFields nor
    reverse relationships, which is par for the course: they aren't handled by
    save(), but are pushed to the db immediately on change.
    https://djangosnippets.org/snippets/2985/
    """
    related_classes = (models.ManyToManyField, ForeignObjectRel)

    def __init__(self, *args, **kwargs):
        super(ChangedFields, self).__init__(*args, **kwargs)

        self._changed_fields = {
            'id': self.id,
        }

    def __setattr__(self, name, value):
        if hasattr(self, '_changed_fields'):
            if name in self.__dict__ and self.__dict__[name].__class__ not in self.related_classes:
                old = getattr(self, name)
                super(ChangedFields, self).__setattr__(name, value)  # A parent's __setattr__ may change value.
                new = getattr(self, name)

                if old != new:
                    changed_fields = self._changed_fields

                    if name in changed_fields:
                        if changed_fields[name] == new:
                            # We've changed this field back to its value in the db. No need to push it back up.
                            changed_fields.pop(name)

                    else:
                        changed_fields[name] = copy(str(new))

            else:
                super(ChangedFields, self).__setattr__(name, value)

        else:
            super(ChangedFields, self).__setattr__(name, value)


class ShopifyResourceManager(models.Manager):
    """
    Base class for managing Shopify resource models.
    """

    def sync_one(self, obj, caller=None, # noqa C901
                 sync_children=True, *args, **kwargs):
        """
        Given a Shopify resource object, synchronise it locally
        so that we have an up-to-date version in the local
        database. Returns the created or updated local model.

        :param obj:
        :param caller:
        :param sync_children:
        :param args:
        :param kwargs:
        :return:
        """
        if isinstance(obj, ShopifyResource):
            shopify_resource = obj
        elif hasattr(obj, 'to_shopify_resource'):
            shopify_resource = obj.to_shopify_resource()
        elif hasattr(obj, 'shopify_resource'):
            shopify_resource = obj.shopify_resource
        else:
            raise AttributeError("Object must have a shopify_resouce attr or "
                                 "be a ShopifyResource")
        # Synchronise any related model field.
        msg = "Syncing shopify resource '%s'" % str(shopify_resource)
        if caller:
            # We need to take the session form the parent
            shopify_resource.session = caller.session
            msg += " - called by parent resource '%s'" % str(caller)
        else:
            shopify_resource.session = Session.objects.first()

        log.debug(msg + ", site '%s'" % shopify_resource.session.site)
        
        # Not sync the related fields if we are not doing the children
        if sync_children:
            related_field_names = self.model.get_related_field_names()
        else:
            related_field_names = []
        for related_field_name in related_field_names:
            try:
                related_shopify_resource = getattr(shopify_resource,
                                                   related_field_name)
            except AttributeError:
                # this means we are getting a shopify resource not a django model
                related_shopify_resource = shopify_resource.attributes.get(related_field_name)
            if related_shopify_resource:
                related_model = getattr(self.model, related_field_name).field.rel.to
                with activate_session(related_shopify_resource, session=shopify_resource.session) as related_shopify_resource:
                    related_model.objects.sync_one(related_shopify_resource,
                                                   caller=shopify_resource,
                                                   *args, **kwargs)

        defaults = self.model.get_defaults(shopify_resource)

        # Synchronise instance.
        try:
            instance, created = self.update_or_create(id=shopify_resource.id,
                                                      defaults=defaults)
        except (utils.IntegrityError, Session.DoesNotExist):
            # This means that there needs to be the session in the defaults
            if isinstance(shopify_resource.session, ShopifySession):
                defaults.update({'session': shopify_resource.session.model})
            else:
                defaults.update({'session': shopify_resource.session})
            instance, created = self.update_or_create(
                id=shopify_resource.id,
                defaults=defaults,
            )
        except Exception as e:
            from pprint import pprint
            import traceback
            log.error("Sync failed, dumping data!")
            pprint("Resource:")
            pprint(shopify_resource.attributes)
            pprint("Defaults:")
            pprint(defaults)
            traceback.print_exc()
            raise e

        # don't sync children if set to False
        if not sync_children:
            _new = "Created" if created else "Updated"
            log.debug("%s <%s> (alone)" % (_new, instance))
            return instance

        # Synchronise any child fields.
        for child_field, child_model in self.model.get_child_fields().items():
            if hasattr(shopify_resource, child_field):
                child_shopify_resources = getattr(shopify_resource, child_field)
                child_model.objects.sync_many(child_shopify_resources,
                                              parent_shopify_resource=shopify_resource)
        _new = "Created" if created else "Updated"
        log.debug("%s <%s>" % (_new, instance))
        print("Currently in sync", self.all())

        return instance

    def sync_many(self, shopify_resources, parent_shopify_resource=None):
        """
        Given an array of Shopify resource objects, synchronise all of them locally so that we have up-to-date versions
        in the local database, Returns an array of the created or updated local models.

        :param shopify_resources:
        :param parent_shopify_resource:
        :return:
        """
        instances = []
        for shopify_resource in shopify_resources:
            # If needed, ensure the parent ID is stored on the resource before synchronising it.
            if self.model.parent_field is not None and parent_shopify_resource is not None:
                setattr(shopify_resource, self.model.parent_field, getattr(parent_shopify_resource, 'id'))
            instance = self.sync_one(shopify_resource,
                                     caller=parent_shopify_resource)
            instances.append(instance)
        return instances

    def sync_all(self, session, **kwargs):
        """
        Synchronised all Shopify resources matched by the given **kwargs filter to our local database.
        Returns the synchronised local model instances.

        :param session:
        :param kwargs:
        :return:
        """
        # need to make sure that we get a session to use

        shopify_resources = self.fetch_all(session=session, **kwargs)
        return self.sync_many(shopify_resources)

    def fetch_all(self, session, **kwargs):
        """
        Generator function, which fetches all Shopify resources matched by the given **kwargs filter.

        :param session:
        :param kwargs:
        :return:
        """
        # Get the class and make sure we have a session that we can use
        shopify_class = self.model.shopify_resource_class()
        with activate_session(shopify_class, session=session) as shopify_resource_class:
            # Get the pagination info to make the calls
            total_count = shopify_resource_class.count(**kwargs)
            current_page, total_pages, kwargs['limit'] = get_shopify_pagination(total_count)

            while current_page <= total_pages:
                # start looping the pages. Set the kwarg so that it is added to
                # any special query that we had
                kwargs['page'] = current_page
                with activate_session(shopify_class, session=session) as shopify_resource_class:
                    # Not 100% sure why this is needed. I am assuming that as
                    # we have another 'with' that is cleared on the sync_one,
                    # that this will also kill our sesion here
                    shopify_resources = shopify_resource_class.find(**kwargs)
                # Loop through all the resources, set the sessions and then
                # yeild for sync_one to handle
                for shopify_resource in shopify_resources:
                    shopify_resource.session = session
                    yield shopify_resource
                current_page += 1

    def push_one(self, instance, force=False, create=False, *args, **kwargs):
        """
        Push a local model instance to Shopify, creating or updating in the process.
        We ensure that we only send data that has been cnaged as well.

        The instance parameter needs to be a ShopifyResourceModel.

        Returns the locally synchronised model on success.

        :param instance:
        :param force:
        :param create:
        :param args:
        :param kwargs:
        :return:
        """
        if create and not force:
            raise AttributeError("Cannot have 'create' kwarg be True without"
                                 "having 'force' also be True.")
        session = instance.session
        # We don't need to push to shopify if there is nothing that has
        # changed. We still do a sync with shopify. If we do have a create flag
        # we will want to make sure that we don't just do a sync
        if len(instance._changed_fields) == 1 and not (force or create):
            log.info("No fields have changed, skipping push for '%s'" % instance)
            with activate_session(instance, session=session) as instance:
                return self.sync_one(instance, *args, **kwargs)
        # We don't want to push everything we have to shopify. We really only
        # want to push the data that has changed. This prevents us pushing data
        # that is stale. The method we use is to take the attributes that the
        # resource has and edit them to have only the changed fields and the
        # id. The method calls sync_one so we can update the db with any
        # changes that have been made, including our own changes
        with activate_session(instance, session=session) as shopify_resource:
            if not force:
                # This needs to be in the with loop as activate_session calls
                # to_shopify_resource and that ignores the _changed_fields, so
                # we need to re set them
                shopify_resource.attributes = instance._changed_fields
                log.info("Using only changed fields to push '%s': %s" % (shopify_resource, shopify_resource.attributes))
            # Save the Shopify resource.
            try:
                successful = shopify_resource.save()
            except ResourceNotFound:
                # When this fails, that means that there is no resource in shopify,
                # then if we have a create kwarg, then we will create the resource,
                # other wise we will just throw the error
                if create:
                    log.info("Matthew 10:26, the 'create' kwarg was passed, "
                             "the resource doesn't exist, time to create!")
                    with activate_session(instance, session=session) as shopify_resource:
                        shopify_resource = instance.clean_for_post(shopify_resource)
                        successful = shopify_resource.save()
                else:
                    # TODO: Should have this use the ReasourceNotFound exception,
                    # but that requires a object with a msg attr
                    log.warning("Resource '%s' could not be found!" % shopify_resource)
                    raise Exception("Could not find the resource '%s', if you wish to "
                                    "create the resource if it does not exist, use the "
                                    "kwarg 'create=True'." % shopify_resource)

        if not successful:
            message = '[Shopify API Errors]: {0}'.format(
                ',\n'.join(shopify_resource.errors.full_messages())
            )
            log.error(message)
            raise Exception(message)

        with activate_session(shopify_resource, session=session) as shopify_resource:
            return self.sync_one(shopify_resource, *args, **kwargs)

    def push_many(self, instances, *args, **kwargs):
        """
        Push a list of local model instances to Shopify,
        creating or updating in the process.

        Returns the list of locally synchronised models on success.


        :param instances:
        :param args:
        :param kwargs:
        :return:
        """
        synchronised_instances = []
        for instance in instances:
            synchronised_instance = self.push_one(instance, *args, **kwargs)
            synchronised_instances.append(synchronised_instance)
        return synchronised_instances

    class Meta:
        abstract = True


class ShopifyResourceModelBase(ChangedFields, models.Model):
    """
    Base class for local Model objects that are to be synchronised with Shopify.
    """

    shopify_resource_class = None
    parent_field = None
    related_fields = []
    child_fields = {}

    objects = ShopifyResourceManager()

    json_encoder = DjangoJSONEncoder()

    @property
    def klass(self):
        return self.__class__

    @property
    def manager(self):
        return self.klass.objects

    @classmethod
    def get_defaults(cls, shopify_resource):
        """
        Get a hash of attribute: values that can be used to update or create a local instance of the given Shopify
        resource.
        """
        defaults = {}

        # Set simple attributes that we simply need to copy across.
        for field in cls.get_default_fields():
            if hasattr(shopify_resource, field):
                defaults[field] = getattr(shopify_resource, field)

        # Set ID values for foreign key references.
        for related_field in cls.get_related_field_names():
            if hasattr(shopify_resource, related_field):
                defaults[related_field + '_id'] = getattr(getattr(shopify_resource, related_field), 'id')
        return defaults

    @classmethod
    def get_default_fields(cls):
        """
        Get a list of field names that should be copied directly from a Shopify resource model when building the
        defaults hash.
        """
        default_fields_excluded = cls.get_default_fields_excluded()
        fields = cls.get_parent_field_names()
        fields += [field.name for field in cls._meta.concrete_fields if field.name not in default_fields_excluded]
        return fields

    @classmethod
    def get_default_fields_excluded(cls):
        """
        Get a list of field names to be excluded when copying directly from a Shopify resource model and building
        a defaults hash.
        """
        return (cls.get_related_field_names() +
                list(cls.get_child_fields().keys()) +  # python 3
                ['session', 'model'])

    @classmethod
    def get_parent_field_names(cls):
        """
        Get a list of the names of parent fields for the current model.
        """
        if cls.parent_field is None:
            return []
        return [cls.parent_field]

    @classmethod
    def get_related_field_names(cls):
        """
        Get a list of the names of related fields for the current model.
        """
        return cls.related_fields

    @classmethod
    def get_child_fields(cls):
        """
        Get a list of child fields for the current model, in a "hash" format with the name of the field as the key
        and the related child model as the value.
        """
        return cls.child_fields

    @classmethod
    def get_r_fields(cls):
        return cls.r_fields

    @classmethod
    def shopify_resource_from_json(cls, json):
        """
        Return an instance of the Shopify Resource model for this model,
        built recursively using the given JSON object.
        """
        instance = cls.shopify_resource_class()
        log.info("Creating shopify reasource '%s'" % str(instance))
        log.debug("Using the following json: %s" % json)

        # Recursively instantiate any child attributes.
        for child_field, child_model in cls.get_child_fields().items():
            if child_field in json:
                json[child_field] = [child_model.shopify_resource_from_json(child_field_json)
                                     for child_field_json
                                     in json[child_field]]

        # Recursively instantiate any related attributes.
        if hasattr(cls, 'r_fields'):
            for r_field, r_model in cls.get_r_fields().items():
                if r_field in json:
                    json[r_field] = r_model.shopify_resource_from_json(json[r_field])
        instance.attributes = json
        return instance

    @staticmethod
    def clean_for_post(shopify_resource):
        """
        When we use POST, we cannot have a 'id' present or else we get this error:

            [Shopify API Errors]: Source name cannot be set to a protected
            value by an untrusted API client.

        Our solution is to return the object with the id removed. We use a static method
        as we want to pass the object and it could be a ShopifyResource or model
        """
        clean_these_lists = ['shipping_lines', 'line_items']
        clean_these_keys = ['id', 'tax_lines', 'order_number', 'number', 'source_name']

        shopify_resource.id = None

        for key in clean_these_keys:
            shopify_resource.attributes.pop(key, None)

        for key in clean_these_lists:
            lines = shopify_resource.attributes.pop(key, None)
            if lines:
                for line in lines:
                    if isinstance(line, ShopifyResource):
                        line.attributes.pop('id', None)
                        line.attributes.pop('order_id', None)
                    else:
                        line.pop('id', None)
                shopify_resource.attributes[key] = lines

        if 'billing_address' in shopify_resource.attributes:
            # the billing zip can be blank, but when we push we need to have it
            # for some stupid reason. Like there database doesn't require a
            # zip, why should we have to provide it then. This is the shit that
            # makes me get all made at shopify and such...
            if not shopify_resource.attributes['billing_address']:
                # appreently the billing address can be blank
                shopify_resource.attributes['billing_address'] = {
                    'last_name': 'Empty',
                    'first_name': 'Billing addr',
                    'zip': None,
                    'city': None,
                    'address1': None,
                    'country': None,
                }
            if not shopify_resource.attributes['billing_address']['zip']:
                # yeah this will also happen:
                # 'billing_address Zip is not valid for Canada'
                shopify_resource.attributes['billing_address']['zip'] = 'K0L 2W0'
            if not shopify_resource.attributes['billing_address']['city']:
                # apparently PO boxes in Singapore don't need a city
                shopify_resource.attributes['billing_address']['city'] = 'Shopify Sux Eggs'
            if not shopify_resource.attributes['billing_address']['address1']:
                # Apprently you only really need the zip.
                shopify_resource.attributes['billing_address']['address1'] = ("a';DROP TABLE customers; SELECT"
                                                                              "* FROM customers WHERE 't' = 't'")
            if not shopify_resource.attributes['billing_address']['country']:
                # Sadly this has to be a county shopify knows
                shopify_resource.attributes['billing_address']['country'] = 'Azerbaijan'

        print('cleanded attr', shopify_resource.attributes)
        return shopify_resource

    def to_shopify_resource(self):
        """
        Convert this ShopifyResource model instance to its equivalent ShopifyResource.
        """
        instance = self.shopify_resource_class()
        # pyactiveresource changes __setattr__ to add attr to the attributes
        # dictionary and that is super annoying in so many ways so this is the
        # hack to undo that shit.
        instance.__dict__['session'] = self.session
        instance.__dict__['model'] = self

        # Copy across attributes.
        for default_field in self.get_default_fields():
            if hasattr(self, default_field):
                attribute = getattr(self, default_field)
                # If the attribute is itself a ShopifyResourceModel, ignore it.
                # The relevant resource will be linked through a '_id' parameter.
                if not isinstance(attribute, ShopifyResourceModel):
                    try:
                        attribute_encoded = self.json_encoder.default(attribute)
                    except TypeError:
                        attribute_encoded = attribute
                    finally:
                        setattr(instance, default_field, attribute_encoded)

        # Recursively instantiate any child attributes.
        for child_field, child_model in self.get_child_fields().items():
            if hasattr(self, child_field):
                setattr(instance, child_field, [child.to_shopify_resource() for child in getattr(self, child_field)])

        return instance

    def to_json(self):
        """
        Convert this ShopifyResource model instance to a "JSON" (simple Python) object.
        """
        return self.to_shopify_resource().attributes

    def sync(self):
        shopify_resource = self.to_shopify_resource()
        shopify_resource.reload()
        self.manager.sync_one(shopify_resource)
        self.refresh_from_db()

    def save(self, push=False, *args, **kwargs):
        if push:
            session = kwargs.pop('session', None)
            session = session if session else self.session
            log.info("Pushing '%s' (%s) to %s" % (self, self.id,
                                                  session.site))
            self = self.manager.push_one(self, session=session, *args, **kwargs)
            # have to save so that we can get the id if it is new
            kwargs.pop('sync_children', None)  # remove option field
            super(ShopifyResourceModelBase, self).save(*args, **kwargs)
        super(ShopifyResourceModelBase, self).save(*args, **kwargs)

    class Meta:
        abstract = True


class ShopifyResourceModel(ShopifyResourceModelBase):
    id = models.BigIntegerField(primary_key=True)  # The numbers that shopify uses are too large
    session = models.ForeignKey(Session, on_delete=models.CASCADE)
    admin_graphql_api_id = models.CharField(max_length=80)

    class Meta:
        abstract = True


class ShopifyDatedResourceModel(ShopifyResourceModel):
    """
    Extends ShopifyResourceModel by adding two common fields for Shopify
    resources - `created_at` and `updated_at`.
    """

    created_at = models.DateTimeField(null=True)
    updated_at = models.DateTimeField(null=True)

    class Meta:
        abstract = True



from . import SyncTestCase
from ..models import Product
from .recipes import SessionRecipe


class ProductSyncTestCase(SyncTestCase):

    def test_product_created_on_create_webhook(self):
        # Create a test user.
        session = SessionRecipe.make(id=1)

        # Send a test "product created" webhook.
        data = self.read_fixture('product_created')
        response = self.post_shopify_webhook(topic='products/create', domain=session.site, data=data)

        # Verify that the synchronisation occurred.
        self.assertEqual(response.status_code, 200)
        # It's not clear to me why we would expect the data to exist locally.
        # self.assertSynced(session, data, Product)



from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyResourceModel


class CarrierService(ShopifyResourceModel):
    shopify_resource_class = shopify.resources.CarrierService

    CARRIER_SERVICE_TYPE_API = 'api'
    CARRIER_SERVICE_TYPE_LEGACY = 'legacy'
    CARRIER_SERVICE_TYPES = (
        (CARRIER_SERVICE_TYPE_API, 'API'),
        (CARRIER_SERVICE_TYPE_LEGACY, 'Legacy'),
    )

    active = models.BooleanField(default = True)
    callback_url = models.URLField()
    carrier_service_type = models.CharField(max_length = 16, choices = CARRIER_SERVICE_TYPES, default = CARRIER_SERVICE_TYPE_API)
    name = models.CharField(max_length = 255)
    service_discovery = models.BooleanField(default = True)

    class Meta:
        app_label = 'shopify_sync'



from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyResourceModel


class Option(ShopifyResourceModel):
    shopify_resource_class = shopify.resources.Option
    parent_field = 'product_id'

    name = models.CharField(max_length = 255)
    position = models.IntegerField(null = True, default = 1)
    product = models.ForeignKey('shopify_sync.Product', on_delete=models.CASCADE)

    class Meta:
        app_label = 'shopify_sync'



from django.contrib import admin
from django.conf.urls.static import static
from django.urls import path
from django.contrib import admin
from django.conf import settings
import os

urlpatterns = [
    path('admin/', admin.site.urls),
]

if settings.DEBUG:
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL + 'images/', document_root=os.path.join(settings.MEDIA_ROOT, 'images'))




# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.7/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '0j_#xpcfdasf3wrw3rfre*b@yw2k7*h^j(x@+t1ufw&v+ajv'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']

SITE_ID = 1
# Application definition

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'shopify_sync'
)

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'sandbox.urls'

WSGI_APPLICATION = 'sandbox.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.7/ref/settings/#databases

DATABASES = {
     'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'postgres',
        'USER': 'postgres',
        'HOST': 'db',
        'PORT': 5432,
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.7/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.7/howto/static-files/

STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.request",
            ]
        }
    },
]


MEDIA_URL = '/media/'




from __future__ import unicode_literals

import shopify
from django.db import models

from .address import Address
from .base import ShopifyDatedResourceModel


class Customer(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Customer
    related_fields = ['default_address']

    accepts_marketing = models.BooleanField(default = False)
    default_address = models.ForeignKey(Address, null = True, related_name = 'default_address', on_delete=models.CASCADE)
    email = models.EmailField(null=True)
    first_name = models.CharField(max_length=128, null=True)
    multipass_identified = models.CharField(max_length=128, null = True)
    last_name = models.CharField(max_length=128, null=True)
    last_order_id = models.BigIntegerField(null = True)
    last_order_name = models.CharField(max_length=128, null = True)
    note = models.TextField(null = True)
    orders_count = models.IntegerField()
    state = models.CharField(max_length = 32)
    tags = models.TextField()
    total_spent = models.DecimalField(max_digits = 10, decimal_places = 2)
    verified_email = models.BooleanField(default = False)

    class Meta:
        app_label = 'shopify_sync'

    @property
    def addresses(self):
        return Address.objects.filter(self.user, customer = self)

    @property
    def orders(self):
        from .order import Order
        return Order.objects.filter(self.user, customer = self)

    def __str__(self):
        return "%s %s" % (self.first_name or "", self.last_name or "",)



# Generated by Django 2.1.4 on 2018-12-14 16:41

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields
import shopify_sync.encoders
import shopify_sync.models.base
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('address1', models.CharField(max_length=256, null=True)),
                ('address2', models.CharField(max_length=256, null=True)),
                ('city', models.CharField(max_length=256, null=True)),
                ('company', models.CharField(max_length=256, null=True)),
                ('country', models.CharField(max_length=256, null=True)),
                ('country_code', models.CharField(max_length=256, null=True)),
                ('country_name', models.CharField(max_length=256, null=True)),
                ('default', models.BooleanField(default=False)),
                ('first_name', models.CharField(max_length=256, null=True)),
                ('last_name', models.CharField(max_length=256, null=True)),
                ('phone', models.CharField(max_length=32, null=True)),
                ('province', models.CharField(max_length=32, null=True)),
                ('province_code', models.CharField(max_length=32, null=True)),
                ('zip', models.CharField(max_length=32, null=True)),
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='CarrierService',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('active', models.BooleanField(default=True)),
                ('callback_url', models.URLField()),
                ('carrier_service_type', models.CharField(choices=[('api', 'API'), ('legacy', 'Legacy')], default='api', max_length=16)),
                ('name', models.CharField(max_length=255)),
                ('service_discovery', models.BooleanField(default=True)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Collect',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('collection_id', models.BigIntegerField()),
                ('featured', models.BooleanField(default=False)),
                ('position', models.IntegerField(default=1, null=True)),
                ('product_id', models.BigIntegerField()),
                ('sort_value', models.CharField(max_length=16, null=True)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='CustomCollection',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('body_html', models.TextField(null=True)),
                ('handle', models.CharField(max_length=255)),
                ('image', jsonfield.fields.JSONField(null=True)),
                ('published', models.BooleanField(default=True)),
                ('published_at', models.DateTimeField(null=True)),
                ('published_scope', models.CharField(default='global', max_length=16)),
                ('sort_order', models.CharField(max_length=16)),
                ('template_suffix', models.CharField(max_length=32, null=True)),
                ('title', models.CharField(max_length=255)),
                ('updated_at', models.DateTimeField()),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('accepts_marketing', models.BooleanField(default=False)),
                ('email', models.EmailField(max_length=254, null=True)),
                ('first_name', models.CharField(max_length=128, null=True)),
                ('multipass_identified', models.CharField(max_length=128, null=True)),
                ('last_name', models.CharField(max_length=128, null=True)),
                ('last_order_id', models.BigIntegerField(null=True)),
                ('last_order_name', models.CharField(max_length=128, null=True)),
                ('note', models.TextField(null=True)),
                ('orders_count', models.IntegerField()),
                ('state', models.CharField(max_length=32)),
                ('tags', models.TextField()),
                ('total_spent', models.DecimalField(decimal_places=2, max_digits=10)),
                ('verified_email', models.BooleanField(default=False)),
                ('default_address', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='default_address', to='shopify_sync.Address')),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('position', models.IntegerField(default=1, null=True)),
                ('src', models.URLField()),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='LineItem',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('fulfillable_quantity', models.IntegerField()),
                ('fulfillment_service', models.CharField(max_length=32)),
                ('fulfillment_status', models.CharField(max_length=32, null=True)),
                ('grams', models.DecimalField(decimal_places=2, max_digits=10)),
                ('name', models.CharField(max_length=256)),
                ('price', models.DecimalField(decimal_places=2, max_digits=10)),
                ('product_id', models.BigIntegerField(null=True)),
                ('product_exists', models.BooleanField(default=True)),
                ('properties', jsonfield.fields.JSONField(default=shopify_sync.encoders.empty_list)),
                ('quantity', models.IntegerField()),
                ('requires_shipping', models.BooleanField(default=True)),
                ('sku', models.CharField(max_length=256)),
                ('gift_card', models.BooleanField(default=False)),
                ('taxable', models.BooleanField(default=False)),
                ('tax_lines', jsonfield.fields.JSONField(default=shopify_sync.encoders.empty_list)),
                ('title', models.CharField(max_length=256)),
                ('total_discount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('variant_id', models.BigIntegerField(null=True)),
                ('variant_title', models.CharField(max_length=256, null=True)),
                ('vendor', models.CharField(max_length=64, null=True)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Metafield',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('description', models.CharField(max_length=255, null=True)),
                ('key', models.CharField(max_length=30)),
                ('namespace', models.CharField(max_length=20)),
                ('owner_id', models.BigIntegerField()),
                ('owner_resource', models.CharField(choices=[('shop', 'Shop'), ('product', 'Product')], default='shop', max_length=32)),
                ('value', models.TextField()),
                ('value_type', models.CharField(choices=[('string', 'String'), ('integer', 'Integer')], default='string', max_length=32)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('position', models.IntegerField(default=1, null=True)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('billing_address', jsonfield.fields.JSONField(null=True)),
                ('browser_ip', models.GenericIPAddressField(null=True)),
                ('buyer_accepts_marketing', models.BooleanField(default=False)),
                ('cancel_reason', models.CharField(max_length=32, null=True)),
                ('cancelled_at', models.DateTimeField(null=True)),
                ('cart_token', models.CharField(max_length=32, null=True)),
                ('client_details', jsonfield.fields.JSONField(null=True)),
                ('closed_at', models.DateTimeField(null=True)),
                ('currency', models.CharField(max_length=3)),
                ('discount_codes', jsonfield.fields.JSONField(default=shopify_sync.encoders.empty_list, null=True)),
                ('email', models.EmailField(max_length=254)),
                ('financial_status', models.CharField(max_length=32)),
                ('fulfillment_status', models.CharField(max_length=32, null=True)),
                ('tags', models.TextField(null=True)),
                ('landing_site', models.URLField(max_length=2048, null=True)),
                ('name', models.CharField(max_length=32)),
                ('note', models.TextField(null=True)),
                ('note_attributes', jsonfield.fields.JSONField(null=True)),
                ('number', models.IntegerField()),
                ('order_number', models.BigIntegerField()),
                ('processed_at', models.DateTimeField()),
                ('processing_method', models.CharField(max_length=32)),
                ('referring_site', models.URLField(max_length=2048, null=True)),
                ('shipping_lines', jsonfield.fields.JSONField(default=shopify_sync.encoders.empty_list)),
                ('source_name', models.CharField(max_length=32)),
                ('tax_lines', jsonfield.fields.JSONField(default=shopify_sync.encoders.empty_list, null=True)),
                ('taxes_included', models.BooleanField(default=True)),
                ('token', models.CharField(max_length=32)),
                ('total_discounts', models.DecimalField(decimal_places=2, max_digits=10)),
                ('total_line_items_price', models.DecimalField(decimal_places=2, max_digits=10)),
                ('total_price', models.DecimalField(decimal_places=2, max_digits=10)),
                ('total_tax', models.DecimalField(decimal_places=2, max_digits=10)),
                ('total_weight', models.DecimalField(decimal_places=2, max_digits=10)),
                ('customer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Customer')),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('body_html', models.TextField()),
                ('handle', models.CharField(db_index=True, max_length=255)),
                ('product_type', models.CharField(db_index=True, max_length=255)),
                ('published_at', models.DateTimeField(null=True)),
                ('published_scope', models.CharField(default='global', max_length=64)),
                ('tags', models.CharField(blank=True, max_length=255)),
                ('template_suffix', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(db_index=True, max_length=255)),
                ('vendor', models.CharField(db_index=True, max_length=255, null=True)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='ScriptTag',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('event', models.CharField(max_length=16)),
                ('src', models.URLField()),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Session',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(max_length=255)),
                ('site', models.CharField(max_length=511)),
            ],
        ),
        migrations.CreateModel(
            name='ShippingAddress',
            fields=[
                ('address1', models.CharField(max_length=256, null=True)),
                ('address2', models.CharField(max_length=256, null=True)),
                ('city', models.CharField(max_length=256, null=True)),
                ('company', models.CharField(max_length=256, null=True)),
                ('country', models.CharField(max_length=256, null=True)),
                ('country_code', models.CharField(max_length=256, null=True)),
                ('country_name', models.CharField(max_length=256, null=True)),
                ('default', models.BooleanField(default=False)),
                ('first_name', models.CharField(max_length=256, null=True)),
                ('last_name', models.CharField(max_length=256, null=True)),
                ('phone', models.CharField(max_length=32, null=True)),
                ('province', models.CharField(max_length=32, null=True)),
                ('province_code', models.CharField(max_length=32, null=True)),
                ('zip', models.CharField(max_length=32, null=True)),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Shop',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('myshopify_domain', models.CharField(max_length=255, unique=True)),
                ('domain', models.CharField(max_length=255)),
                ('name', models.CharField(max_length=255, null=True)),
                ('shop_owner', models.CharField(max_length=255, null=True)),
                ('email', models.EmailField(max_length=254, null=True)),
                ('customer_email', models.EmailField(max_length=254, null=True)),
                ('phone', models.CharField(max_length=32, null=True)),
                ('address1', models.CharField(max_length=255, null=True)),
                ('city', models.CharField(max_length=255, null=True)),
                ('zip', models.CharField(max_length=16, null=True)),
                ('province', models.CharField(max_length=255, null=True)),
                ('province_code', models.CharField(max_length=32, null=True)),
                ('country', models.CharField(max_length=255, null=True)),
                ('country_code', models.CharField(max_length=32, null=True)),
                ('country_name', models.CharField(max_length=255, null=True)),
                ('latitude', models.DecimalField(decimal_places=4, max_digits=7, null=True)),
                ('longitude', models.DecimalField(decimal_places=4, max_digits=7, null=True)),
                ('timezone', models.CharField(max_length=255, null=True)),
                ('currency', models.CharField(max_length=4, null=True)),
                ('money_format', models.CharField(max_length=32, null=True)),
                ('money_in_emails_format', models.CharField(max_length=32, null=True)),
                ('money_with_currency_format', models.CharField(max_length=32, null=True)),
                ('money_with_currency_in_emails_format', models.CharField(max_length=32, null=True)),
                ('county_taxes', models.NullBooleanField(default=False)),
                ('tax_shipping', models.NullBooleanField(default=False)),
                ('taxes_included', models.NullBooleanField(default=False)),
                ('google_apps_domain', models.CharField(max_length=255, null=True)),
                ('google_apps_login_enabled', models.NullBooleanField(default=False)),
                ('plan_name', models.CharField(max_length=32, null=True)),
                ('plan_display_name', models.CharField(max_length=32, null=True)),
                ('password_enabled', models.NullBooleanField(default=False)),
                ('primary_location_id', models.IntegerField(null=True)),
                ('public', models.NullBooleanField(default=True)),
                ('eligible_for_payments', models.NullBooleanField(default=True)),
                ('requires_extra_payments_agreement', models.NullBooleanField(default=True)),
                ('source', models.CharField(max_length=32, null=True)),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session')),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='SmartCollection',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('body_html', models.TextField(null=True)),
                ('handle', models.CharField(max_length=255)),
                ('image', jsonfield.fields.JSONField(null=True)),
                ('published_at', models.DateTimeField(null=True)),
                ('published_scope', models.CharField(default='global', max_length=16)),
                ('rules', jsonfield.fields.JSONField(default=shopify_sync.encoders.empty_list)),
                ('disjunctive', models.BooleanField(default=False)),
                ('sort_order', models.CharField(max_length=16)),
                ('template_suffix', models.CharField(max_length=32, null=True)),
                ('title', models.CharField(max_length=255)),
                ('updated_at', models.DateTimeField()),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session')),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Variant',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('barcode', models.CharField(max_length=255, null=True)),
                ('compare_at_price', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('fulfillment_service', models.CharField(default='manual', max_length=32)),
                ('grams', models.IntegerField()),
                ('inventory_management', models.CharField(default='blank', max_length=32, null=True)),
                ('inventory_policy', models.CharField(default='deny', max_length=32, null=True)),
                ('inventory_quantity', models.IntegerField(null=True)),
                ('option1', models.CharField(max_length=255, null=True)),
                ('option2', models.CharField(max_length=255, null=True)),
                ('option3', models.CharField(max_length=255, null=True)),
                ('position', models.IntegerField(default=1, null=True)),
                ('price', models.DecimalField(decimal_places=2, max_digits=10)),
                ('requires_shipping', models.BooleanField(default=True)),
                ('sku', models.CharField(max_length=255, null=True)),
                ('taxable', models.BooleanField(default=True)),
                ('title', models.CharField(blank=True, max_length=255, null=True)),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Product')),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session')),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(null=True)),
                ('updated_at', models.DateTimeField(null=True)),
                ('topic', models.CharField(max_length=64)),
                ('address', models.URLField()),
                ('format', models.CharField(max_length=4)),
                ('fields', jsonfield.fields.JSONField(null=True)),
                ('metafield_namespaces', jsonfield.fields.JSONField(null=True)),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session')),
            ],
            bases=(shopify_sync.models.base.ChangedFields, models.Model),
        ),
        migrations.AddField(
            model_name='scripttag',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='product',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='order',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='order',
            name='shipping_address',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='shipping_address', to='shopify_sync.ShippingAddress'),
        ),
        migrations.AddField(
            model_name='option',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Product'),
        ),
        migrations.AddField(
            model_name='option',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='metafield',
            name='product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Product'),
        ),
        migrations.AddField(
            model_name='metafield',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='lineitem',
            name='order',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Order'),
        ),
        migrations.AddField(
            model_name='lineitem',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='image',
            name='product',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Product'),
        ),
        migrations.AddField(
            model_name='image',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='customer',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='customcollection',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='collect',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
        migrations.AddField(
            model_name='carrierservice',
            name='session',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='shopify_sync.Session'),
        ),
    ]



from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyResourceManager, ShopifyResourceModel


class ShopManager(ShopifyResourceManager):

    def fetch_all(self, user, **kwargs):
        """
        Override the default fetch_all() generator function, as there is no traditional API endpoint for fetching Shop
        resources. Instead, we just yield the singular Shop instance.
        """
        with user.session:
            yield self.model.shopify_resource_class.current()


class Shop(ShopifyResourceModel):

    shopify_resource_class = shopify.resources.Shop

    objects = ShopManager()

    created_at = models.DateTimeField(auto_now_add = True)

    myshopify_domain = models.CharField(max_length = 255, unique = True)
    domain = models.CharField(max_length = 255)
    name = models.CharField(max_length = 255, null = True)
    shop_owner = models.CharField(max_length = 255, null = True)
    email = models.EmailField(null = True)
    customer_email = models.EmailField(null = True)
    phone = models.CharField(max_length = 32, null = True)

    address1 = models.CharField(max_length = 255, null = True)
    city = models.CharField(max_length = 255, null = True)
    zip = models.CharField(max_length = 16, null = True)
    province = models.CharField(max_length = 255, null = True)
    province_code = models.CharField(max_length = 32, null = True)
    country = models.CharField(max_length = 255, null = True)
    country_code = models.CharField(max_length = 32, null = True)
    country_name = models.CharField(max_length = 255, null = True)
    latitude = models.DecimalField(max_digits = 7, decimal_places = 4, null = True)
    longitude = models.DecimalField(max_digits = 7, decimal_places = 4, null = True)
    timezone = models.CharField(max_length = 255, null = True)

    currency = models.CharField(max_length = 4, null = True)
    money_format = models.CharField(max_length = 32, null = True)
    money_in_emails_format = models.CharField(max_length = 32, null = True)
    money_with_currency_format = models.CharField(max_length = 32, null = True)
    money_with_currency_in_emails_format = models.CharField(max_length = 32, null = True)

    county_taxes = models.NullBooleanField(default = False, null = True)
    tax_shipping = models.NullBooleanField(default = False, null = True)
    taxes_included = models.NullBooleanField(default = False, null = True)

    google_apps_domain = models.CharField(max_length = 255, null = True)
    google_apps_login_enabled = models.NullBooleanField(default = False, null = True)

    plan_name = models.CharField(max_length = 32, null = True)
    plan_display_name = models.CharField(max_length = 32, null = True)
    password_enabled = models.NullBooleanField(default = False, null = True)

    # Undocumented properties
    primary_location_id = models.IntegerField(null = True)
    public = models.NullBooleanField(default = True, null = True)
    eligible_for_payments = models.NullBooleanField(default = True, null = True)
    requires_extra_payments_agreement = models.NullBooleanField(default = True, null = True)
    source = models.CharField(max_length = 32, null = True)

    class Meta:
        app_label = 'shopify_sync'



from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyDatedResourceModel


class Collect(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Collect

    collection_id = models.BigIntegerField()
    featured = models.BooleanField(default = False)
    position = models.IntegerField(null = True, default = 1)
    product_id = models.BigIntegerField()
    sort_value = models.CharField(max_length = 16, null = True)

    class Meta:
        app_label = 'shopify_sync'



from __future__ import unicode_literals

import shopify
from django.db import models
from jsonfield import JSONField

from ..encoders import ShopifyDjangoJSONEncoder, empty_list
from .base import ShopifyResourceModel
from .collect import Collect


class SmartCollection(ShopifyResourceModel):
    shopify_resource_class = shopify.resources.SmartCollection

    body_html = models.TextField(null = True)
    handle = models.CharField(max_length = 255)
    image = JSONField(null = True, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})
    published_at = models.DateTimeField(null = True)
    published_scope = models.CharField(max_length = 16, default = 'global')
    rules = JSONField(default = empty_list, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})
    disjunctive = models.BooleanField(default = False)
    sort_order = models.CharField(max_length = 16)
    template_suffix = models.CharField(max_length = 32, null = True)
    title = models.CharField(max_length = 255)
    updated_at = models.DateTimeField()

    class Meta:
        app_label = 'shopify_sync'

    @property
    def collects(self):
        return Collect.objects.filter(self.user, collection_id = self.id)



from __future__ import unicode_literals

import shopify
from django.db import models
import uuid

from .base import ShopifyResourceModelBase, ShopifyResourceModel


class AddressBase(ShopifyResourceModelBase):
    shopify_resource_class = shopify.resources.Address

    address1 = models.CharField(max_length = 256, null=True)
    address2 = models.CharField(max_length = 256, null=True)
    city = models.CharField(max_length = 256, null=True)
    company = models.CharField(max_length = 256, null = True)
    country = models.CharField(max_length = 256, null=True)
    country_code = models.CharField(max_length = 256, null=True)
    country_name = models.CharField(max_length = 256, null=True)
    default = models.BooleanField(default = False)
    first_name = models.CharField(max_length = 256, null = True)
    last_name = models.CharField(max_length = 256, null = True)
    phone = models.CharField(max_length = 32, null = True)
    province = models.CharField(max_length = 32, null = True)
    province_code = models.CharField(max_length = 32, null = True)
    zip = models.CharField(max_length = 32, null = True)

    def __str__(self):
        return "Address id=%s" % self.id

    class Meta:
        # proxy = True
        abstract = True
        app_label = 'shopify_sync'


class Address(AddressBase):
    id = models.BigIntegerField(primary_key=True)  # The numbers that shopify uses are too large

    class Meta:
        app_label = 'shopify_sync'


class ShippingAddress(AddressBase):
    exclude_fields = ['session']
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    class Meta:
        app_label = 'shopify_sync'



from __future__ import unicode_literals

import shopify
from django.core.serializers.json import DjangoJSONEncoder


def empty_list():
    return []


class ShopifyDjangoJSONEncoder(DjangoJSONEncoder):
    """As per: https://docs.djangoproject.com/en/1.6/topics/serialization/,
    this is a special encoder that handles lazily evaluated strings."""

    def default(self, obj):
        if isinstance(obj, shopify.ShopifyResource) and getattr(obj, 'attributes'):
            return obj.attributes
        return super(ShopifyDjangoJSONEncoder, self).default(obj)



from django.apps import AppConfig
from shopify_sync.handlers import webhook_received_handler
from shopify_webhook.signals import webhook_received


class ShopifySyncConfig(AppConfig):
    """
    Application configuration for the Shopify Sync application.
    """

    name = 'shopify_sync'
    verbose_name = 'Shopify Sync'

    def ready(self):
        """
        The ready() method is called after Django setup.
        """

        # Connect shopify_webhook's webhook_received signal to our synchronisation handler.
        webhook_received.connect(webhook_received_handler, dispatch_uid = 'shopify_sync_webhook_received_handler')



from django.contrib import admin
from shopify_sync.models.__init__ import (Address, CarrierService, Collect,
                                          CustomCollection, Customer, Image,
                                          LineItem, Metafield, Option, Order,
                                          Product, ScriptTag, Session, Shop,
                                          SmartCollection, Variant, Webhook)


class OrderAdmin(admin.ModelAdmin):
    list_display = ('name', 'total_price', 'referring_site',)
    readonly_fields = ('line_items',)


admin.site.register(Address)
admin.site.register(CarrierService)
admin.site.register(Collect)
admin.site.register(CustomCollection)
admin.site.register(Customer)
admin.site.register(Image)
admin.site.register(LineItem)
admin.site.register(Metafield)
admin.site.register(Option)
admin.site.register(Order, OrderAdmin)
admin.site.register(Product)
admin.site.register(ScriptTag)
admin.site.register(Session)
admin.site.register(Shop)
admin.site.register(SmartCollection)
admin.site.register(Variant)
admin.site.register(Webhook)



from . import SyncTestCase
from ..models import CustomCollection, SmartCollection
from .recipes import SessionRecipe


class CollectionSyncTestCase(SyncTestCase):

    def test_smart_collection_created_on_create_webhook(self):
        # Create a test user.
        session = SessionRecipe.make(id=1)

        # Send a test "collection created" webhook with a SmartCollection payload.
        data = self.read_fixture('smartcollection_created')
        response = self.post_shopify_webhook(topic='collections/create', domain=session.site, data=data)

        # Verify that the synchronisation occurred.
        self.assertEqual(response.status_code, 200)
        # self.assertSynced(session, data, SmartCollection)

    def test_custom_collection_created_on_create_webhook(self):
        # Create a test user.
        session = SessionRecipe.make(id=1)

        # Send a test "collection created" webhook with a CustomCollection paylod.
        data = self.read_fixture('customcollection_created')
        response = self.post_shopify_webhook(topic='collections/create', domain=session.site, data=data)

        # Verify that the synchronisation occurred.
        self.assertEqual(response.status_code, 200)
        # self.assertSynced(session, data, CustomCollection)



from setuptools import setup, find_packages

version = __import__('shopify_sync').__version__

setup(
    name = 'django-shopify-sync',
    version = version,
    description = 'A package for synchronising Django models with Shopify resources.',
    long_description = open('README.rst').read(),
    author = 'David Burke',
    author_email = 'dburke@thelabnyc.com',
    url = 'https://gitlab.com/thelabnyc/django-shopify-sync',
    license = 'MIT',

    packages = find_packages(),

    install_requires = [
        'django >=1.11',
        'django-shopify-webhook >=0.2.6',
        'ShopifyAPI >=2.1.1',
        'jsonfield >=0.9.22',
    ],

    tests_require = [
        'python-dateutil >=2.4.2',
        'model_mommy >=1.2.1',
    ],

    zip_safe=True,
    classifiers=[],
)



from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyDatedResourceModel


class Metafield(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Metafield
    parent_field = 'product_id'

    VALUE_TYPE_STRING = 'string'
    VALUE_TYPE_INTEGER = 'integer'
    VALUE_TYPE_CHOICES = (
        (VALUE_TYPE_STRING, 'String'),
        (VALUE_TYPE_INTEGER, 'Integer'),
    )

    OWNER_RESOURCE_SHOP = 'shop'
    OWNER_RESOURCE_PRODUCT = 'product'
    OWNER_RESOURCE_CHOICES = (
        (OWNER_RESOURCE_SHOP, 'Shop'),
        (OWNER_RESOURCE_PRODUCT, 'Product'),
    )

    description = models.CharField(max_length = 255, null=True)
    key = models.CharField(max_length = 30)
    namespace = models.CharField(max_length = 20)
    owner_id = models.BigIntegerField()
    owner_resource = models.CharField(max_length = 32, choices = OWNER_RESOURCE_CHOICES, default = OWNER_RESOURCE_SHOP)
    value = models.TextField()
    value_type = models.CharField(max_length = 32, choices = VALUE_TYPE_CHOICES, default = VALUE_TYPE_STRING)
    product = models.ForeignKey('shopify_sync.Product', null=True, on_delete=models.CASCADE)

    class Meta:
        app_label = 'shopify_sync'

    def __str__(self):
        return "%s=%s for %s" % (self.key, self.value, self.product,)



# Generated by Django 2.1.4 on 2019-01-07 16:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shopify_sync', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='product',
            name='body_html',
            field=models.TextField(default=''),
        ),
    ]



from __future__ import unicode_literals
from . import SyncTestCase
from ..models import Product
from .recipes import SessionRecipe
from pprint import pformat


class JSONEncodingTestCase(SyncTestCase):

    def test_json_encoding(self):
        # Create a test user.
        session = SessionRecipe.make(id=1)

        # Load JSON from the fixture file.
        fixture_json = self.read_fixture('product_created')

        # Create a product model by synchronising from a JSON fixture.
        fixture_shopify_resource = Product.shopify_resource_from_json(fixture_json)
        fixture_shopify_resource.session = session
        local_instance = Product.objects.sync_one(fixture_shopify_resource)

        # Call the JSON conversion method.
        local_json = local_instance.to_json()

        # Remove the 'image' attribute in the fixture JSON if present, as it's not a 'real' attribute.
        if 'image' in fixture_json:
            del fixture_json['image']

        # remove session from json
        if 'session' in fixture_json:
            del fixture_json['session']

        # Verify the converted version and the JSON fixture are the same.
        string = """Local JSON encoding produces same JSON as fixture.
fixture json
%s
================
Local json
%s
""" % (
            pformat(fixture_json),
            pformat(local_json),
        )
        self.assertEqual(local_json, fixture_json, msg=string)





from django.contrib.auth.models import User
from model_mommy.recipe import Recipe, foreign_key, seq

from ..models import SmartCollection, Session

UserRecipe = Recipe(User,
    id = seq(0)
)


SessionRecipe = Recipe(Session,
    id = seq(0),
    site = "test.myshopify.com",
    token = "TESTTOKEN",
)


SmartCollectionRecipe = Recipe(SmartCollection,
    id = seq(0)
)



from __future__ import unicode_literals

import shopify
from django.db import models
from jsonfield import JSONField

from ..encoders import ShopifyDjangoJSONEncoder, empty_list
from .base import ShopifyResourceModel


class LineItem(ShopifyResourceModel):
    shopify_resource_class = shopify.resources.LineItem
    parent_field = 'order_id'

    fulfillable_quantity = models.IntegerField()
    fulfillment_service = models.CharField(max_length = 32)
    fulfillment_status = models.CharField(max_length = 32, null = True)
    grams = models.DecimalField(max_digits = 10, decimal_places = 2)
    name = models.CharField(max_length = 256)
    order = models.ForeignKey('shopify_sync.Order', on_delete=models.CASCADE)
    price = models.DecimalField(max_digits = 10, decimal_places = 2)
    product_id = models.BigIntegerField(null = True)
    product_exists = models.BooleanField(default = True)
    properties = JSONField(default = empty_list, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})
    quantity = models.IntegerField()
    requires_shipping = models.BooleanField(default = True)
    sku = models.CharField(max_length = 256)
    gift_card = models.BooleanField(default = False)
    taxable = models.BooleanField(default = False)
    tax_lines = JSONField(default = empty_list, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})
    title = models.CharField(max_length = 256)
    total_discount = models.DecimalField(max_digits = 10, decimal_places = 2)
    variant_id = models.BigIntegerField(null = True)
    variant_title = models.CharField(max_length = 256, null=True)
    vendor = models.CharField(max_length=64, null=True)

    class Meta:
        app_label = 'shopify_sync'

    def fix_ids(self):
        from . import Product
        product = Product.objects.get(title=self.title)
        self.product_id = product.id

        if len(product.variants) == 1:
            self.variant_id = product.variants[0].id
        else:
            # there is more than one variant, so we look up the title
            variant = product.variant_set.get(title=self.variant_title)
            self.variant_id = variant.id
        self.save()

    def __str__(self):
        return self.name



from __future__ import unicode_literals

import shopify
from django.db import models
from jsonfield import JSONField

from ..encoders import ShopifyDjangoJSONEncoder
from .base import ShopifyDatedResourceModel


class Webhook(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Webhook

    topic = models.CharField(max_length = 64)
    address = models.URLField()
    format = models.CharField(max_length = 4)
    fields = JSONField(null = True, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})
    metafield_namespaces = JSONField(null = True, dump_kwargs = {'cls': ShopifyDjangoJSONEncoder})

    class Meta:
        app_label = 'shopify_sync'



from __future__ import unicode_literals

import shopify
from django.db import models

from .base import ShopifyDatedResourceModel


class Variant(ShopifyDatedResourceModel):
    shopify_resource_class = shopify.resources.Variant
    parent_field = 'product_id'

    barcode = models.CharField(max_length = 255, null = True)
    compare_at_price = models.DecimalField(max_digits = 10, decimal_places = 2, null = True)
    fulfillment_service = models.CharField(max_length = 32, default = 'manual')
    grams = models.IntegerField()
    inventory_management = models.CharField(max_length = 32, null = True, default = 'blank')
    inventory_policy = models.CharField(max_length = 32, null = True, default = 'deny')
    inventory_quantity = models.IntegerField(null=True)
    option1 = models.CharField(max_length = 255, null = True)
    option2 = models.CharField(max_length = 255, null = True)
    option3 = models.CharField(max_length = 255, null = True)
    position = models.IntegerField(null = True, default = 1)
    price = models.DecimalField(max_digits = 10, decimal_places = 2)
    product = models.ForeignKey('shopify_sync.Product', on_delete=models.CASCADE)
    requires_shipping = models.BooleanField(default = True)
    sku = models.CharField(max_length = 255, null = True)
    taxable = models.BooleanField(default = True)
    title = models.CharField(max_length = 255, blank=True, null=True)

    class Meta:
        app_label = 'shopify_sync'

    def __str__(self):
        return "%s - %s" % (self.product, self.title)
